/*
Update channel and fetch command from talk back

Copyright 2018, The MathWorks, Inc.

*/

#include <WiFiS3.h>

#define LED_BUILTIN 13

// Update these with values suitable for your network.
#include "arduino_secrets.h" 
///////please enter your sensitive data in the Secret tab/arduino_secrets.h
char ssid[] = SECRET_SSID;        // your network SSID (name)
char pass[] = SECRET_PASS;    // your network password (use for WPA, or use as key for WEP)
int status = WL_IDLE_STATUS;     // the WiFi radio's status


const char *API = "XWH7ZA117WD2VX5D";  // CHANGE ME
const char *HOST = "api.thingspeak.com";
const int PORT = 80;
const char *field1 = "field1";

WiFiClient client;

unsigned long myTalkBackID = 50325;
const char *myTalkBackKey = "70K7GUSGS6IYS4RU";

// Some values to send to ThingSpeak
int number1 = 0;
int number2 = random(0,100);
int number3 = random(0,100);
int number4 = random(0,100);


void setup_wifi() {
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  // check for the WiFi module:
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    // don't continue
    while (true);
  }

  String fv = WiFi.firmwareVersion();
  if (fv < WIFI_FIRMWARE_LATEST_VERSION) {
    Serial.println("Please upgrade the firmware");
  }

  // attempt to connect to WiFi network:
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to WPA SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network:
    if(strlen(pass)) status = WiFi.begin(ssid, pass);
    else status = WiFi.begin(ssid);

    // wait 1 second for connection:
    delay(1000);
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


//In the setup function, initialize the LED, start the serial monitor and 
//establishing a connection to the local WiFi network.
void setup() {
  pinMode(LED_BUILTIN, OUTPUT);  // Set up LED

  Serial.begin(115200);  // Initialize serial
  setup_wifi();

  Serial.println("");
  Serial.println("\nConnected.");
}


//In the main loop, start by createint the POST message with the correct parameters. Send the POST request, verify the result,
//and check for a TalkBack command. Wait 60 seconds and check the queue again.
void loop() {
  // Create the TalkBack URI
  String tbURI = String("/update");

  number1=random(0,1000); number2=random(0,1000); number3=random(0,1000); number4=random(0,1000);
  // Create the message body for the POST out of the values
  String postMessage =  String("field1=") + String(number1) +
                        String("&field2=") + String(number2) +
                        String("&field3=") + String(number3) +
                        String("&field4=") + String(number4) +
                        String("&api_key=") + String(API) +
                        String("&talkback_key=") + String(myTalkBackKey);

  // Make a string for any commands in the queue
  String newCommand = String();

  // Make the POST to ThingSpeak
  int x = httpPOST(tbURI, postMessage, newCommand);

  // Check the result
  if (x == 200) {
    Serial.println("checking queue...");
    // check for a command returned from TalkBack
    if (newCommand.length() != 0) {

      Serial.print("  Latest command from queue: ");
      Serial.println(newCommand);

      if (newCommand.indexOf("LED_ON")!=(-1)) {
        digitalWrite(LED_BUILTIN, HIGH);
      }

      if (newCommand.indexOf("LED_OFF")!=-1) {
        digitalWrite(LED_BUILTIN, LOW);
      }
    } else {
      Serial.println("  Nothing new.");
    }
  } 
  else {
    Serial.println("Problem checking queue. HTTP error code " + String(x));
  }
  delay(30000);  // Wait 60 seconds to check queue again
}

int getSensorData() {
  return random(1000); // Replace with
}

//Use the httpPOST function to read the next TalkBack command.


// General function to POST to ThingSpeak
int httpPOST(String uri, String postMessage, String &response){

  bool connectSuccess = false;
  connectSuccess = client.connect("api.thingspeak.com",80);

  if(!connectSuccess){
      return -301;   
  }
  
  postMessage += "&headers=false";
  
  String Headers =  String("POST ") + uri + String(" HTTP/1.1\r\n") +
                    String("Host: api.thingspeak.com\r\n") +
                    String("Content-Type: application/x-www-form-urlencoded\r\n") +
                    String("Connection: close\r\n") +
                    String("Content-Length: ") + String(postMessage.length()) +
                    String("\r\n\r\n");

  Serial.print(Headers);
  Serial.println(postMessage);

  client.print(Headers);
  client.print(postMessage);

  long startWaitForResponseAt = millis();
  while(client.available() == 0 && millis() - startWaitForResponseAt < 5000){
      delay(100);
  }

  if(client.available() == 0){       
    return -304; // Didn't get server response in time
  }

  if(!client.find("HTTP/1.1 ")){ //const_cast<char *>
      return -303; // Couldn't parse response (didn't find HTTP/1.1)
  }

  int status = client.readStringUntil(' ').toInt(); // parseInt();

  if(status != 200){
    return status;
  }

  if(!client.find("\n\r\n")){
    return -303;
  }

  String temp2String = String(client.readString());
  response = temp2String;
  
  return status;   
}